@@ -42,13 +42,13 @@ namespace cura
 
 bool FffPolygonGenerator::generateAreas(SliceDataStorage& storage, MeshGroup* meshgroup, TimeKeeper& timeKeeper)
 {
-    if (!sliceModel(meshgroup, timeKeeper, storage)) 
+    if (!sliceModel(meshgroup, timeKeeper, storage))
     {
         return false;
     }
-    
+
     slices2polygons(storage, timeKeeper);
-    
+
     return true;
 }
 
@@ -71,7 +71,7 @@ unsigned int FffPolygonGenerator::getDraftShieldLayerCount(const unsigned int to
 bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeeper, SliceDataStorage& storage) /// slices the model
 {
     Progress::messageProgressStage(Progress::Stage::SLICING, &timeKeeper);
-    
+
     storage.model_min = meshgroup->min();
     storage.model_max = meshgroup->max();
     storage.model_size = storage.model_max - storage.model_min;
@@ -89,8 +89,7 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
         logError("Layer height %i is disallowed.\n", layer_thickness);
         return false;
     }
-    int initial_slice_z = initial_layer_thickness - layer_thickness / 2;
-    int slice_layer_count = (storage.model_max.z - initial_slice_z) / layer_thickness + 1;
+    int slice_layer_count = (storage.model_max.z - initial_layer_thickness) / layer_thickness + 2;
     if (slice_layer_count <= 0) //Model is shallower than layer_height_0, so not even the first layer is sliced. Return an empty model then.
     {
         return true; //This is NOT an error state!
@@ -100,7 +99,7 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
     for(unsigned int mesh_idx = 0; mesh_idx < meshgroup->meshes.size(); mesh_idx++)
     {
         Mesh& mesh = meshgroup->meshes[mesh_idx];
-        Slicer* slicer = new Slicer(&mesh, initial_slice_z, layer_thickness, slice_layer_count, mesh.getSettingBoolean("meshfix_keep_open_polygons"), mesh.getSettingBoolean("meshfix_extensive_stitching"));
+        Slicer* slicer = new Slicer(&mesh, initial_layer_thickness, layer_thickness, slice_layer_count, mesh.getSettingBoolean("meshfix_keep_open_polygons"), mesh.getSettingBoolean("meshfix_extensive_stitching"));
         slicerList.push_back(slicer);
         /*
         for(SlicerLayer& layer : slicer->layers)
@@ -172,13 +171,13 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
         for (unsigned int layer_nr = 0; layer_nr < meshStorage.layers.size(); layer_nr++)
         {
             SliceLayer& layer = meshStorage.layers[layer_nr];
-            meshStorage.layers[layer_nr].printZ += 
+            meshStorage.layers[layer_nr].printZ =
                 getSettingInMicrons("layer_height_0")
-                - initial_slice_z;
+                + layer_nr * layer_thickness;
             if (has_raft)
             {
                 ExtruderTrain* train = storage.meshgroup->getExtruderTrain(getSettingAsIndex("adhesion_extruder_nr"));
-                layer.printZ += 
+                layer.printZ +=
                     Raft::getTotalThickness(storage)
                     + train->getSettingInMicrons("raft_airgap")
                     - train->getSettingInMicrons("layer_0_z_overlap"); // shift all layers (except 0) down
@@ -249,7 +248,9 @@ void FffPolygonGenerator::slices2polygons(SliceDataStorage& storage, TimeKeeper&
     // brim depends on the first layer not being empty
     // only remove empty layers if we haven't generate support, because then support was added underneath the model.
     //   for some materials it's better to print on support than on the buildplate.
-    removeEmptyFirstLayers(storage, getSettingInMicrons("layer_height"), storage.print_layer_count); // changes storage.print_layer_count!
+    if (getSettingBoolean("remove_empty_first_layers")) {
+        removeEmptyFirstLayers(storage, getSettingInMicrons("layer_height"), storage.print_layer_count); // changes storage.print_layer_count!
+    }
     if (storage.print_layer_count == 0)
     {
         log("Stopping process because there are no non-empty layers.\n");
@@ -261,26 +262,34 @@ void FffPolygonGenerator::slices2polygons(SliceDataStorage& storage, TimeKeeper&
     // handle helpers
     storage.primeTower.generatePaths(storage);
     storage.primeTower.subtractFromSupport(storage);
-    
+
     logDebug("Processing ooze shield\n");
     processOozeShield(storage);
 
     logDebug("Processing draft shield\n");
     processDraftShield(storage);
 
-    logDebug("Processing platform adhesion\n");
-    processPlatformAdhesion(storage);
+    // This catches a special case in which the models are in the air, and then
+    // the adhesion mustn't be calculated.
+    if (!isEmptyLayer(storage, 0) || storage.primeTower.enabled)
+    {
+        log("Processing platform adhesion\n");
+        processPlatformAdhesion(storage);
+    }
 
+    logDebug("Processing gaps\n");
     processOutlineGaps(storage);
     processPerimeterGaps(storage);
 
+    logDebug("Meshes post-processing\n");
     // meshes post processing
     for (SliceMeshStorage& mesh : storage.meshes)
     {
         processDerivedWallsSkinInfill(mesh);
     }
 
-    // generate gradual suppport
+    logDebug("Processing gradual support\n");
+    // generate gradual support
     AreaSupport::generateSupportInfillFeatures(storage);
 }
 
@@ -293,18 +302,18 @@ void FffPolygonGenerator::processBasicWallsSkinInfill(SliceDataStorage& storage,
     {
         processInfillMesh(storage, mesh_order_idx, mesh_order);
     }
-    
+
     // TODO: make progress more accurate!!
     // note: estimated time for     insets : skins = 22.953 : 48.858
     std::vector<double> walls_vs_skin_timing({22.953, 48.858});
     ProgressStageEstimator* mesh_inset_skin_progress_estimator = new ProgressStageEstimator(walls_vs_skin_timing);
-    
+
     inset_skin_progress_estimate.nextStage(mesh_inset_skin_progress_estimator); // the stage of this function call
-    
+
     ProgressEstimatorLinear* inset_estimator = new ProgressEstimatorLinear(mesh_layer_count);
     mesh_inset_skin_progress_estimator->nextStage(inset_estimator);
-    
-    
+
+
     // walls
     unsigned int processed_layer_count = 0;
 #pragma omp parallel for default(none) shared(mesh_layer_count, storage, mesh, inset_skin_progress_estimate, processed_layer_count) schedule(dynamic)
@@ -562,7 +571,7 @@ void FffPolygonGenerator::processInfillMesh(SliceDataStorage& storage, unsigned
                 }
             }
         }
-        
+
         layer.parts.clear();
         for (PolygonsPart& part : new_parts)
         {
@@ -574,9 +583,9 @@ void FffPolygonGenerator::processInfillMesh(SliceDataStorage& storage, unsigned
         if (layer.parts.size() > 0 || (mesh.getSettingAsSurfaceMode("magic_mesh_surface_mode") != ESurfaceMode::NORMAL && layer.openPolyLines.size() > 0) )
         {
             mesh.layer_nr_max_filled_layer = layer_idx; // last set by the highest non-empty layer
-        } 
+        }
     }
-    
+
 }
 
 void FffPolygonGenerator::processDerivedWallsSkinInfill(SliceMeshStorage& mesh)
@@ -664,40 +673,40 @@ void FffPolygonGenerator::processInsets(const SliceDataStorage& storage, SliceMe
     }
 }
 
+bool FffPolygonGenerator::isEmptyLayer(SliceDataStorage& storage, const unsigned int layer_idx)
+{
+    if (storage.support.generated && layer_idx < storage.support.supportLayers.size())
+    {
+        SupportLayer& support_layer = storage.support.supportLayers[layer_idx];
+        if (!support_layer.support_infill_parts.empty() || !support_layer.support_bottom.empty() || !support_layer.support_roof.empty())
+        {
+            return false;
+        }
+    }
+    for (SliceMeshStorage& mesh : storage.meshes)
+    {
+        SliceLayer& layer = mesh.layers[layer_idx];
+        if (mesh.getSettingAsSurfaceMode("magic_mesh_surface_mode") != ESurfaceMode::NORMAL && layer.openPolyLines.size() > 0)
+        {
+            return false;
+        }
+        for (const SliceLayerPart& part : layer.parts)
+        {
+            if (part.print_outline.size() > 0)
+            {
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
 void FffPolygonGenerator::removeEmptyFirstLayers(SliceDataStorage& storage, const int layer_height, unsigned int& total_layers)
 {
     int n_empty_first_layers = 0;
     for (unsigned int layer_idx = 0; layer_idx < total_layers; layer_idx++)
-    { 
-        bool layer_is_empty = true;
-        if (storage.support.generated && layer_idx < storage.support.supportLayers.size())
-        {
-            SupportLayer& support_layer = storage.support.supportLayers[layer_idx];
-            if (!support_layer.support_infill_parts.empty() || !support_layer.support_bottom.empty() || !support_layer.support_roof.empty())
-            {
-                layer_is_empty = false;
-                break;
-            }
-        }
-        for (SliceMeshStorage& mesh : storage.meshes)
-        {
-            SliceLayer& layer = mesh.layers[layer_idx];
-            if (mesh.getSettingAsSurfaceMode("magic_mesh_surface_mode") != ESurfaceMode::NORMAL && layer.openPolyLines.size() > 0)
-            {
-                layer_is_empty = false;
-                break;
-            }
-            for (const SliceLayerPart& part : layer.parts)
-            {
-                if (part.print_outline.size() > 0)
-                {
-                    layer_is_empty = false;
-                    break;
-                }
-            }
-        }
-        
-        if (layer_is_empty) 
+    {
+        if (isEmptyLayer(storage, layer_idx))
         {
             n_empty_first_layers++;
         } else
@@ -705,7 +714,7 @@ void FffPolygonGenerator::removeEmptyFirstLayers(SliceDataStorage& storage, cons
             break;
         }
     }
-    
+
     if (n_empty_first_layers > 0)
     {
         log("Removing %d layers because they are empty\n", n_empty_first_layers);
@@ -738,8 +747,8 @@ void FffPolygonGenerator::removeEmptyFirstLayers(SliceDataStorage& storage, cons
  */
 void FffPolygonGenerator::processSkinsAndInfill(const SliceDataStorage& storage, SliceMeshStorage& mesh, unsigned int layer_nr, bool process_infill)
 {
-    if (mesh.getSettingAsSurfaceMode("magic_mesh_surface_mode") == ESurfaceMode::SURFACE) 
-    { 
+    if (mesh.getSettingAsSurfaceMode("magic_mesh_surface_mode") == ESurfaceMode::SURFACE)
+    {
         return;
     }
 
@@ -917,13 +926,13 @@ void FffPolygonGenerator::processFuzzyWalls(SliceMeshStorage& mesh)
             {
                 // generate points in between p0 and p1
                 PolygonRef result = results.newPoly();
-                
+
                 int64_t dist_left_over = rand() % (min_dist_between_points / 2); // the distance to be traversed on the line before making the first new point
                 Point* p0 = &poly.back();
                 for (Point& p1 : poly)
                 { // 'a' is the (next) new point between p0 and p1
                     Point p0p1 = p1 - *p0;
-                    int64_t p0p1_size = vSize(p0p1);    
+                    int64_t p0p1_size = vSize(p0p1);
                     int64_t dist_last_point = dist_left_over + p0p1_size * 2; // so that p0p1_size - dist_last_point evaulates to dist_left_over - p0p1_size
                     for (int64_t p0pa_dist = dist_left_over; p0pa_dist < p0p1_size; p0pa_dist += min_dist_between_points + rand() % range_random_point_dist)
                     {
@@ -935,7 +944,7 @@ void FffPolygonGenerator::processFuzzyWalls(SliceMeshStorage& mesh)
                         dist_last_point = p0pa_dist;
                     }
                     dist_left_over = p0p1_size - dist_last_point;
-                    
+
                     p0 = &p1;
                 }
                 while (result.size() < 3 )
