@@ -93,7 +93,7 @@ void SettingsBase::setSettingInheritBase(std::string key, const SettingsBaseVirt
 }
 
 
-std::string SettingsBase::getSettingString(std::string key) const
+const std::string& SettingsBase::getSettingString(const std::string& key) const
 {
     auto value_it = setting_values.find(key);
     if (value_it != setting_values.end())
@@ -112,7 +112,8 @@ std::string SettingsBase::getSettingString(std::string key) const
 
     cura::logError("Trying to retrieve unregistered setting with no value given: '%s'\n", key.c_str());
     std::exit(-1);
-    return "";
+    static std::string empty_string; // use static object rather than "" to avoid compilation warning
+    return empty_string;
 }
 
 void SettingsMessenger::setSetting(std::string key, std::string value)
@@ -126,14 +127,14 @@ void SettingsMessenger::setSettingInheritBase(std::string key, const SettingsBas
 }
 
 
-std::string SettingsMessenger::getSettingString(std::string key) const
+const std::string& SettingsMessenger::getSettingString(const std::string& key) const
 {
     return parent->getSettingString(key);
 }
 
 int SettingsBaseVirtual::getSettingAsIndex(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atoi(value.c_str());
 }
 
@@ -144,12 +145,18 @@ int SettingsBaseVirtual::getSettingAsExtruderNr(std::string key) const
     {
         extruder_nr = getSettingAsIndex("extruder_nr");
     }
+    const int max_extruders = getSettingAsCount("machine_extruder_count");
+    if (extruder_nr >= max_extruders)
+    {
+        cura::logWarning("Trying to get extruder %s=%i, while there are only %i extruders.\n", key.c_str(), extruder_nr, max_extruders);
+        return 0;
+    }
     return extruder_nr;
 }
 
 int SettingsBaseVirtual::getSettingAsCount(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atoi(value.c_str());
 }
 
@@ -166,7 +173,7 @@ unsigned int SettingsBaseVirtual::getSettingAsLayerNumber(std::string key) const
 
 double SettingsBaseVirtual::getSettingInMillimeters(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str());
 }
 
@@ -177,19 +184,19 @@ coord_t SettingsBaseVirtual::getSettingInMicrons(std::string key) const
 
 double SettingsBaseVirtual::getSettingInAngleDegrees(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str());
 }
 
 double SettingsBaseVirtual::getSettingInAngleRadians(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str()) / 180.0 * M_PI;
 }
 
 bool SettingsBaseVirtual::getSettingBoolean(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "on")
         return true;
     if (value == "yes")
@@ -202,43 +209,43 @@ bool SettingsBaseVirtual::getSettingBoolean(std::string key) const
 
 double SettingsBaseVirtual::getSettingInDegreeCelsius(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str());
 }
 
 double SettingsBaseVirtual::getSettingInMillimetersPerSecond(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return std::max(0.0, atof(value.c_str()));
 }
 
 double SettingsBaseVirtual::getSettingInCubicMillimeters(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str());
 }
 
 double SettingsBaseVirtual::getSettingInPercentage(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return std::max(0.0, atof(value.c_str()));
 }
 
 double SettingsBaseVirtual::getSettingAsRatio(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return atof(value.c_str()) / 100.0;
 }
 
 double SettingsBaseVirtual::getSettingInSeconds(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     return std::max(0.0, atof(value.c_str()));
 }
 
 DraftShieldHeightLimitation SettingsBaseVirtual::getSettingAsDraftShieldHeightLimitation(const std::string key) const
 {
-    const std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "full")
     {
         return DraftShieldHeightLimitation::FULL;
@@ -298,7 +305,7 @@ FMatrix3x3 SettingsBaseVirtual::getSettingAsPointMatrix(std::string key) const
 {
     FMatrix3x3 ret;
 
-    std::string value_string = getSettingString(key);
+    const std::string& value_string = getSettingString(key);
     if (value_string.empty())
     {
         return ret; // standard matrix ([1,0,0],[0,1,0],[0,0,1])
@@ -338,7 +345,7 @@ FMatrix3x3 SettingsBaseVirtual::getSettingAsPointMatrix(std::string key) const
 
 EGCodeFlavor SettingsBaseVirtual::getSettingAsGCodeFlavor(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "Griffin")
         return EGCodeFlavor::GRIFFIN;
     else if (value == "UltiGCode")
@@ -360,7 +367,7 @@ EGCodeFlavor SettingsBaseVirtual::getSettingAsGCodeFlavor(std::string key) const
 
 EFillMethod SettingsBaseVirtual::getSettingAsFillMethod(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "lines")
         return EFillMethod::LINES;
     if (value == "grid")
@@ -375,6 +382,8 @@ EFillMethod SettingsBaseVirtual::getSettingAsFillMethod(std::string key) const
         return EFillMethod::QUARTER_CUBIC;
     if (value == "triangles")
         return EFillMethod::TRIANGLES;
+    if (value == "trihexagon")
+        return EFillMethod::TRIHEXAGON;
     if (value == "concentric")
         return EFillMethod::CONCENTRIC;
     if (value == "concentric_3d")
@@ -390,7 +399,7 @@ EFillMethod SettingsBaseVirtual::getSettingAsFillMethod(std::string key) const
 
 EPlatformAdhesion SettingsBaseVirtual::getSettingAsPlatformAdhesion(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "brim")
         return EPlatformAdhesion::BRIM;
     if (value == "raft")
@@ -402,7 +411,7 @@ EPlatformAdhesion SettingsBaseVirtual::getSettingAsPlatformAdhesion(std::string
 
 ESupportType SettingsBaseVirtual::getSettingAsSupportType(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "everywhere")
         return ESupportType::EVERYWHERE;
     if (value == "buildplate")
@@ -412,7 +421,7 @@ ESupportType SettingsBaseVirtual::getSettingAsSupportType(std::string key) const
 
 EZSeamType SettingsBaseVirtual::getSettingAsZSeamType(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "random")
         return EZSeamType::RANDOM;
     if (value == "shortest")
@@ -426,7 +435,7 @@ EZSeamType SettingsBaseVirtual::getSettingAsZSeamType(std::string key) const
 
 EZSeamCornerPrefType SettingsBaseVirtual::getSettingAsZSeamCornerPrefType(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "z_seam_corner_none")
         return EZSeamCornerPrefType::Z_SEAM_CORNER_PREF_NONE;
     if (value == "z_seam_corner_inner")
@@ -440,7 +449,7 @@ EZSeamCornerPrefType SettingsBaseVirtual::getSettingAsZSeamCornerPrefType(std::s
 
 ESurfaceMode SettingsBaseVirtual::getSettingAsSurfaceMode(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "normal")
         return ESurfaceMode::NORMAL;
     if (value == "surface")
@@ -452,7 +461,7 @@ ESurfaceMode SettingsBaseVirtual::getSettingAsSurfaceMode(std::string key) const
 
 FillPerimeterGapMode SettingsBaseVirtual::getSettingAsFillPerimeterGapMode(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "nowhere")
     {
         return FillPerimeterGapMode::NOWHERE;
@@ -466,7 +475,7 @@ FillPerimeterGapMode SettingsBaseVirtual::getSettingAsFillPerimeterGapMode(std::
 
 CombingMode SettingsBaseVirtual::getSettingAsCombingMode(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "off")
     {
         return CombingMode::OFF;
@@ -484,7 +493,7 @@ CombingMode SettingsBaseVirtual::getSettingAsCombingMode(std::string key) const
 
 SupportDistPriority SettingsBaseVirtual::getSettingAsSupportDistPriority(std::string key) const
 {
-    std::string value = getSettingString(key);
+    const std::string& value = getSettingString(key);
     if (value == "xy_overrides_z")
     {
         return SupportDistPriority::XY_OVERRIDES_Z;
@@ -496,10 +505,24 @@ SupportDistPriority SettingsBaseVirtual::getSettingAsSupportDistPriority(std::st
     return SupportDistPriority::XY_OVERRIDES_Z;
 }
 
+SlicingTolerance SettingsBaseVirtual::getSettingAsSlicingTolerance(std::string key) const
+{
+    const std::string& value = getSettingString(key);
+    if (value == "inclusive")
+    {
+        return SlicingTolerance::INCLUSIVE;
+    }
+    if (value == "exclusive")
+    {
+        return SlicingTolerance::EXCLUSIVE;
+    }
+    return SlicingTolerance::MIDDLE;
+}
+
 std::vector<int> SettingsBaseVirtual::getSettingAsIntegerList(std::string key) const
 {
     std::vector<int> result;
-    std::string value_string = getSettingString(key);
+    const std::string& value_string = getSettingString(key);
     if (!value_string.empty()) {
         // we're looking to match one or more integer values separated by commas and surrounded by square brackets
         // note that because the QML RegExpValidator only stops unrecognised characters being input
